'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _mapValues3 = require('lodash/mapValues');

var _mapValues4 = _interopRequireDefault(_mapValues3);

var _forEach3 = require('lodash/forEach');

var _forEach4 = _interopRequireDefault(_forEach3);

var _size3 = require('lodash/size');

var _size4 = _interopRequireDefault(_size3);

exports.combineMiddlewareWithActionHandlers = combineMiddlewareWithActionHandlers;
exports.default = composeMiddleware;

var _createMiddleware = require('./createMiddleware');

var _createMiddleware2 = _interopRequireDefault(_createMiddleware);

var _checkForNonFunctions = require('./checkForNonFunctions');

var _checkForNonFunctions2 = _interopRequireDefault(_checkForNonFunctions);

var _addCreationStack = require('./addCreationStack');

var _addCreationStack2 = _interopRequireDefault(_addCreationStack);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param middlewares an array of one or more middlewares.
 * @returns a middleware that calls all of the given middlewares in sequence.
 */
function naiveComposeMiddleware(middlewares) {
  if (middlewares.length === 0) return function (store) {
    return function (dispatch) {
      return dispatch;
    };
  };
  if (middlewares.length === 1) return middlewares[0];
  return function (store) {
    return function (next) {
      var handleAction = middlewares.reduceRight(function (next, handler) {
        return handler(store)(next);
      }, next);
      if (process.env.NODE_ENV !== 'production') handleAction = (0, _addCreationStack2.default)(handleAction, 'middleware');
      return handleAction;
    };
  };
}

function composeActionHandlers(actionHandlers) {
  return (0, _createMiddleware2.default)((0, _mapValues4.default)(actionHandlers, naiveComposeMiddleware));
}

function combineMiddlewareWithActionHandlers() {
  for (var _len = arguments.length, middlewares = Array(_len), _key = 0; _key < _len; _key++) {
    middlewares[_key] = arguments[_key];
  }

  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions2.default)(middlewares, 'middlewares');

  var optimizedMiddlewares = [];
  var actionHandlers = {};

  middlewares.forEach(function (middleware) {
    if (middleware.actionHandlers) {
      (0, _forEach4.default)(middleware.actionHandlers, function (actionHandler, type) {
        (actionHandlers[type] || (actionHandlers[type] = [])).push(actionHandler);
      });
    } else {
      if ((0, _size4.default)(actionHandlers)) {
        optimizedMiddlewares.push(composeActionHandlers(actionHandlers));
        actionHandlers = {};
      }
      optimizedMiddlewares.push(middleware);
    }
  });
  if ((0, _size4.default)(actionHandlers)) optimizedMiddlewares.push(composeActionHandlers(actionHandlers));

  return optimizedMiddlewares;
}

function composeMiddleware() {
  return naiveComposeMiddleware(combineMiddlewareWithActionHandlers.apply(undefined, arguments));
}