'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _mapValues3 = require('lodash/mapValues');

var _mapValues4 = _interopRequireDefault(_mapValues3);

var _forEach3 = require('lodash/forEach');

var _forEach4 = _interopRequireDefault(_forEach3);

var _size3 = require('lodash/size');

var _size4 = _interopRequireDefault(_size3);

exports.combineReducersWithActionHandlers = combineReducersWithActionHandlers;
exports.default = composeReducers;

var _createReducer = require('./createReducer');

var _createReducer2 = _interopRequireDefault(_createReducer);

var _addCreationStack = require('./addCreationStack');

var _addCreationStack2 = _interopRequireDefault(_addCreationStack);

var _checkForNonFunctions = require('./checkForNonFunctions');

var _checkForNonFunctions2 = _interopRequireDefault(_checkForNonFunctions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param reducers an array of one or more reducers.
 * @returns a reducer that calls all of the given reducers in sequence.
 */
function naiveComposeReducers(reducers) {
  if (reducers.length === 0) return function (state) {
    return state;
  };
  if (reducers.length === 1) return reducers[0];
  var initialState = reducers.reduce(function (state, reducer) {
    return reducer(state, { type: '' });
  }, undefined);
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments[1];
    return reducers.reduce(function (state, reducer) {
      return reducer(state, action);
    }, state);
  };
}

function composeActionHandlers(initialState, actionHandlers) {
  return (0, _createReducer2.default)(initialState, (0, _mapValues4.default)(actionHandlers, naiveComposeReducers));
}

function combineReducersWithActionHandlers() {
  for (var _len = arguments.length, reducers = Array(_len), _key = 0; _key < _len; _key++) {
    reducers[_key] = arguments[_key];
  }

  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions2.default)(reducers, 'reducers');

  var optimizedReducers = [];
  var initialState = undefined;
  var actionHandlers = {};

  reducers.forEach(function (reducer) {
    if (reducer.actionHandlers) {
      initialState = reducer(initialState, { type: '' });
      (0, _forEach4.default)(reducer.actionHandlers, function (actionHandler, type) {
        (actionHandlers[type] || (actionHandlers[type] = [])).push(actionHandler);
      });
    } else {
      if ((0, _size4.default)(actionHandlers)) {
        optimizedReducers.push(composeActionHandlers(initialState, actionHandlers));
        actionHandlers = {};
        initialState = undefined;
      }
      optimizedReducers.push(reducer);
    }
  });
  if ((0, _size4.default)(actionHandlers)) optimizedReducers.push(composeActionHandlers(initialState, actionHandlers));

  return optimizedReducers;
}

function composeReducers() {
  var result = naiveComposeReducers(combineReducersWithActionHandlers.apply(undefined, arguments));
  if (process.env.NODE_ENV !== 'production') result = (0, _addCreationStack2.default)(result, 'reducer');
  return result;
}